/*
 * Copyright (c) 2008 - 2012, Andy Bierman, All Rights Reserved.
 * Copyright (c) 2012 - 2020, YumaWorks, Inc., All Rights Reserved.
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *

*** Generated by yangdump-sdk

    User SIL module
    module example
    revision 2020-03-03
    namespace http://netconfcentral.org/ns/example
    Created: 2020-03-05T23:22:42Z
    CLI parameters:
        format uc
        indent 4
        log-level debug3
        module example
        output u_example.c
        unified true

 */

#include <xmlstring.h>

#include "procdefs.h"
#include "agt.h"
#include "agt_action.h"
#include "agt_cb.h"
#include "agt_rpc.h"
#include "agt_sil_lib.h"
#include "agt_timer.h"
#include "agt_util.h"
#include "agt_val.h"
#include "cfg.h"
#include "dlq.h"
#include "ncx.h"
#include "ncx_feature.h"
#include "ncxmod.h"
#include "ncxtypes.h"
#include "rpc.h"
#include "status.h"
#include "val.h"
#include "val_child.h"
#include "val_util.h"
#include "xml_val.h"
#include "u_example.h"
#include "y_example.h"


/* put your static variables here */


/********************************************************************
* FUNCTION u_example_system_host_name_edit
*
* Edit database object callback
* Path: leaf /system/host-name
* Add object instrumentation in COMMIT phase.
*
* INPUTS:
*     see agt/agt_cb.h for details
*
* RETURNS:
*     error status
********************************************************************/
status_t
    u_example_system_host_name_edit (ses_cb_t *scb,
                                     rpc_msg_t *msg,
                                     agt_cbtyp_t cbtyp,
                                     op_editop_t editop,
                                     val_value_t *newval,
                                     val_value_t *curval)
{
    status_t res = NO_ERR;
    const xmlChar *newval_val = (newval) ? VAL_STRING(newval) : 0;

    if (LOGDEBUG) {
        log_debug("\nEnter u_example_system_host_name_edit callback for %s phase",
            agt_cbtype_name(cbtyp));
    }

    /* not used in this example */
    (void)scb;
    (void)msg;
    (void)curval;

    switch (cbtyp) {
    case AGT_CB_VALIDATE:
        /* description-stmt validation here */
        break;
    case AGT_CB_APPLY:
        /* database manipulation done here */
        break;
    case AGT_CB_COMMIT:
        /* device instrumentation done here */
        switch (editop) {
        case OP_EDITOP_LOAD:
            break;
        case OP_EDITOP_REPLACE:
            break;
        case OP_EDITOP_CREATE:
        case OP_EDITOP_MERGE:

             /* Force Rollback if the value is not acceptable */
            if (newval &&
                !xml_strcmp(newval_val, (const xmlChar *)"not-supported"))  {

                log_info("\n\n Value is not supported for %s editop, host-name=%s",
                         op_editop_name(editop),
                         newval_val);

                /* Validation failed if a value is not supported */
                res = ERR_NCX_OPERATION_NOT_SUPPORTED;
            } else {
                /* Run device instrumentation here  */
            }

            break;
        case OP_EDITOP_DELETE:
            break;
        default:
            res = SET_ERROR(ERR_INTERNAL_VAL);
        }
        break;
    case AGT_CB_ROLLBACK:
        /* undo device instrumentation here */
        break;
    default:
        res = SET_ERROR(ERR_INTERNAL_VAL);
    }
    return res;

} /* u_example_system_host_name_edit */


/********************************************************************
* FUNCTION u_example_system_domain_search_edit
*
* Edit database object callback
* Path: leaf-list /system/domain-search
* Add object instrumentation in COMMIT phase.
*
* INPUTS:
*     see agt/agt_cb.h for details
*
* RETURNS:
*     error status
********************************************************************/
status_t
    u_example_system_domain_search_edit (ses_cb_t *scb,
                                         rpc_msg_t *msg,
                                         agt_cbtyp_t cbtyp,
                                         op_editop_t editop,
                                         val_value_t *newval,
                                         val_value_t *curval)
{
    status_t res = NO_ERR;
    const xmlChar *newval_val = (newval) ? VAL_STRING(newval) : 0;
    const xmlChar *curval_val = (curval) ? VAL_STRING(curval) : 0;

    (void)scb;
    (void)msg;
    (void)curval;
    (void)newval;
    (void)curval_val;
    (void)newval_val;

    if (LOGDEBUG) {
        log_debug("\nEnter u_example_system_domain_search_edit callback for %s phase",
            agt_cbtype_name(cbtyp));
    }

    switch (cbtyp) {
    case AGT_CB_VALIDATE:
        /* description-stmt validation here */
        break;
    case AGT_CB_APPLY:
        /* database manipulation done here */
        break;
    case AGT_CB_COMMIT:
        /* device instrumentation done here */
        switch (editop) {
        case OP_EDITOP_LOAD:
            break;
        case OP_EDITOP_MERGE:
            break;
        case OP_EDITOP_REPLACE:
            break;
        case OP_EDITOP_CREATE:
            break;
        case OP_EDITOP_DELETE:
            break;
        default:
            res = SET_ERROR(ERR_INTERNAL_VAL);
        }
        break;
    case AGT_CB_ROLLBACK:
        /* undo device instrumentation here */
        break;
    default:
        res = SET_ERROR(ERR_INTERNAL_VAL);
    }
    return res;

} /* u_example_system_domain_search_edit */


/********************************************************************
* FUNCTION u_example_system_edit
*
* Edit database object callback
* Path: container /system
* Add object instrumentation in COMMIT phase.
*
* INPUTS:
*     see agt/agt_cb.h for details
*
* RETURNS:
*     error status
********************************************************************/
status_t
    u_example_system_edit (ses_cb_t *scb,
                           rpc_msg_t *msg,
                           agt_cbtyp_t cbtyp,
                           op_editop_t editop,
                           val_value_t *newval,
                           val_value_t *curval)
{
    status_t res = NO_ERR;

    (void)scb;
    (void)msg;
    (void)curval;
    (void)newval;

    if (LOGDEBUG) {
        log_debug("\nEnter u_example_system_edit callback for %s phase",
            agt_cbtype_name(cbtyp));
    }

    switch (cbtyp) {
    case AGT_CB_VALIDATE:
        /* description-stmt validation here */
        break;
    case AGT_CB_APPLY:
        /* database manipulation done here */
        break;
    case AGT_CB_COMMIT:
        /* device instrumentation done here */
        switch (editop) {
        case OP_EDITOP_LOAD:
            break;
        case OP_EDITOP_MERGE:
            break;
        case OP_EDITOP_REPLACE:
            break;
        case OP_EDITOP_CREATE:
            break;
        case OP_EDITOP_DELETE:
            break;
        default:
            res = SET_ERROR(ERR_INTERNAL_VAL);
        }
        break;
    case AGT_CB_ROLLBACK:
        /* undo device instrumentation here */
        break;
    default:
        res = SET_ERROR(ERR_INTERNAL_VAL);
    }
    return res;

} /* u_example_system_edit */


/********************************************************************
* FUNCTION u_example_init
*
* initialize the example server instrumentation library
*
* INPUTS:
*    modname == requested module name
*    revision == requested version (NULL for any)
*
* RETURNS:
*     error status
********************************************************************/
status_t
    u_example_init (const xmlChar *modname,
                    const xmlChar *revision)
{
    status_t res = NO_ERR;
    ncx_module_t *example_mod = NULL;

    example_mod = ncx_find_module(modname, revision);
    if (example_mod == NULL) {
        return ERR_NCX_OPERATION_FAILED;
    }

    /* put your module initialization code here */

    return res;

} /* u_example_init */


/********************************************************************
* FUNCTION u_example_init2
*
* SIL init phase 2: non-config data structures
* Called after running config is loaded
*
* RETURNS:
*     error status
********************************************************************/
status_t u_example_init2 (void)
{
    status_t res = NO_ERR;

    /* put your init2 code here */

    return res;

} /* u_example_init2 */


/********************************************************************
* FUNCTION u_example_cleanup
*    cleanup the server instrumentation library
*
********************************************************************/
void u_example_cleanup (void)
{

    /* put your cleanup code here */

} /* u_example_cleanup */

/* END SIL u_example.c */
